﻿/* **************************************************************************
 * File Name:  ModuleInterface.cs
 * Version  :  1.1
 * Author   :  Philip Allen
 * Date     :  8 May 2009
 * Updates  :  None
 ****************************************************************************/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Resources;
using System.Windows;
using System.Threading;
using SerialComms;

namespace RelayInterface
{
    class ModuleInterface
    {
        #region PrivateMemebers
        private int[] mcLowerRelay = new int[8];
        private int[] mcUpperRelay = new int[8];
        private int mDataPortBDirControl;
        private int mDataPortCDirControl;
        private int mDataPortDDirControl;
        SerialInterface SerialCommPort;

        #endregion

        public enum DirectionState {InputState, OutputState};

        #region Constructors

        //---------------------------------------------------------------------
        public ModuleInterface(SerialInterface CommPort)
        {
            SerialCommPort = CommPort;
        }
        #endregion

        //---------------------------------------------------------------------
        public void SendCommand(char ControlChar, byte Value)
        {
            // As only two characters are sent a small buffer is enough
            byte[] sbuff = new byte [2];
            sbuff[0] = (byte)ControlChar;
            sbuff[1] = Value;
            SerialCommPort.Send(sbuff);
            Thread.Sleep(5);
        }

        //---------------------------------------------------------------------
        // Used to open all relays for non multplexed versions 
        public void ClearRelays()
        {
            byte RData;

            RData = (byte)0;
            //set All relay lines low
            SendCommand('C', RData);
        }

        //---------------------------------------------------------------------
        // Used to open all relays for multplexed versions 
        public void ClearRelays(int NumRelays)
        {
            byte uRData;

            // Set Latch enable off
            uRData = (byte)0;

            // Clear Relay control words
            for (int i = 1; i <= 3; i++)
            {
                mcLowerRelay[i] = 0;
                mcUpperRelay[i] = 0;
            }

            //set All relay lines low
            SendCommand('C', uRData);
            Thread.Sleep(10);

            // This for the multiplexed version of modules
            if (NumRelays > 24)
            {
                //Set all latch ouputs low
                for (int i = 1; i <= 6; i++)
                {
                    uRData = (byte)(1 << i);
                    SendCommand('C', uRData);

                    //Return bank clock low for next access
                    uRData = (byte)0;
                    SendCommand('C', uRData);
                }
                // Set Latch enable back to on
                uRData = (byte)1;
                SendCommand('C', uRData);
            }
        }

        //---------------------------------------------------------------------
        /// <summary>
        /// Creates string to be sent to serial port for closing a relay
        /// </summary>
        /// <param name="RelayPosition"></param>
        public void CloseRelay(int RelayPosition)
        {
            int RelBank;
            int RelNum;

            RelBank = (RelayPosition / (int)(8));
            if ((RelNum = RelayPosition % 8) == 0)
            {
                RelNum = 8;
                RelBank = (RelayPosition / (int)(8));
            }
            else
            {
                RelBank = (RelayPosition / (int)(8)) + 1;
            }

            // Calculate Control character
            switch (RelBank)
            {
                case 1:
                    mcLowerRelay[1] = mcLowerRelay[1] | (1 << (RelNum - 1));
                    SendCommand('C', (byte)mcLowerRelay[1]);
                    break;
                case 2:
                    mcLowerRelay[2] = mcLowerRelay[2] | (1 << (RelNum - 1));
                    SendCommand('E', (byte)mcLowerRelay[2]);
                    break;
                case 3: ;
                    mcLowerRelay[3] = mcLowerRelay[3] | (1 << (RelNum - 1));
                    SendCommand('J', (byte)mcLowerRelay[3]);
                    break;
                default: break;
            }
        }

        //---------------------------------------------------------------------
        /// <summary>
        /// Creates string to be sent to serial port for opening a relay
        /// </summary>
        /// <param name="RelayPosition"></param>
        public void OpenRelay(int RelayPosition)
        {
            int RelBank;
            int RelNum;

            RelBank = (RelayPosition / (int)(8));
            if ((RelNum = RelayPosition % 8) == 0)
            {
                RelNum = 8;
                RelBank = (RelayPosition / (int)(8));
            }
            else
            {
                RelBank = (RelayPosition / (int)(8)) + 1;
            }


            switch (RelBank)
            {
                case 1:
                    mcLowerRelay[1] = mcLowerRelay[1] & (~(1 << (RelNum - 1)));
                    SendCommand('C', (byte)mcLowerRelay[1]);
                    break;
                case 2:
                    mcLowerRelay[2] = mcLowerRelay[2] & (~(1 << (RelNum - 1)));
                    SendCommand('E', (byte)mcLowerRelay[2]);
                    break;
                case 3: ;
                    mcLowerRelay[3] = mcLowerRelay[3] & (~(1 << (RelNum - 1)));
                    SendCommand('J', (byte)mcLowerRelay[3]);
                    break;
                default: break;
            }
        }

        //---------------------------------------------------------------------
        /// <summary>
        /// Reads the data port
        /// A read command is first sent to the port to be read and then the
        /// value read back from the serial port
        /// </summary>
        /// <param name="Port"></param>
        /// <param name="PortValue"></param>
        /// <returns></returns>
        public int ReadDataPortVal(char Port, out byte[] PortValue)
        {
            if (Port == 'B')
            {
                SendCommand('A', (byte)0);
            }
            else
            {
                if (Port == 'C')
                {
                    SendCommand('D', (byte)0xF);
                }
            }

            SerialCommPort.Read(out PortValue);

            return 0;
        }

        //---------------------------------------------------------------------
        /// <summary>
        /// Sets the PIC ports to outputs or inputs
        /// Mask = bit high to change bit, low to not change bit
        /// </summary>
        /// <param name="Port"></param>
        /// <param name="Dir"></param>
        /// <param name="Mask"></param>
        public void SetPortStates(char Port, DirectionState Dir, int Mask)
        {
            int DirMask;

            if (Dir == DirectionState.InputState)
            {
                DirMask = 0xFF;
            }
            else
            {
                DirMask = 0;
            }
            /* The variable mDataPortXDirControl is modified each time this function is called
             * The value of Mask defines which pins are to be changed and DirMask is set by
             * the value of port direction.*/
            switch (Port)
            {
                case 'B':
                    mDataPortBDirControl = ((~Mask) & mDataPortBDirControl) | (Mask & DirMask);
                    SendCommand('B', (byte)mDataPortBDirControl); //Set B port
                    break;
                case 'C':
                    mDataPortCDirControl &= ((~Mask) & mDataPortCDirControl) | (Mask & DirMask);
                    SendCommand('E', (byte)mDataPortCDirControl); //Set C port
                    break;
                case 'D':
                    mDataPortDDirControl &= ((~Mask) & mDataPortDDirControl) | (Mask & DirMask);
                    SendCommand('H', (byte)mDataPortDDirControl); //Set D port
                    break;
            }
            Thread.Sleep(10);
        }
    }
}
