﻿/* **************************************************************************
 * File Name:  RelayControlForm.cs
 * Version  :  1.1
 * Author   :  Philip Allen
 * Date     :  8 May 2009
 * Updates  :  None
 ****************************************************************************/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using SerialComms;
using RelayInterface;
using System.IO.Ports;

namespace RelayFormsApplication
{
    public partial class RelayControlForm : Form
    {
        ModuleInterface RelayControl;

        /// <summary>
        /// Port being connected to.
        /// </summary>
        private string PortName = null;

        //---------------------------------------------------------------------
        /// <summary>
        /// Constructor - Sets up Combo box for any ports available
        /// </summary>
        public RelayControlForm()
        {
            InitializeComponent();
            // Load Combo box with Comm ports
            ConnectComboBox.Items.Add("Choose Port...");
            ConnectComboBox.SelectedIndex = 0;

            // Add a list of available ports.
            foreach (string PortName in SerialPort.GetPortNames())
            {
                ConnectComboBox.Items.Add(PortName);
            }

            // Set basic form controls
            EnableControls(false);
            ConnectComboBox.Enabled = true;

            txtRelayNumber.Text = "";
        }
 
        //---------------------------------------------------------------------
        /// <summary>
        /// Reads the only data port and outputs to text box.
        /// </summary>
        public void ReceiveDataHandler()
        {
            byte[] MsgBytes = new byte[2];

            RelayControl.ReadDataPortVal('B', out MsgBytes);
            this.txtPortReading.Text = "0x" + ConvertStringToHex(MsgBytes);
        }

        //---------------------------------------------------------------------
        /// <summary>
        /// Converts the only byte used in this app to hexadecimal value
        /// </summary>
        /// <param name="MsgString"></param>
        /// <returns></returns>
        private char ConvertStringToHex(byte[] MsgString)
        {
            byte LowerByte = (byte)(MsgString[0] >> 4);
            if (LowerByte < 10)
            {
                return (char)('0' + LowerByte);
            }
            else
            {
                return (char)('A' +(LowerByte -10));
            }
        }
        
        //---------------------------------------------------------------------
        /// <summary>
        /// Called when form first loads.  If no serial port set, clears Combo
        /// box index
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void RelayControlForm_Load(object sender, EventArgs e)
        {
            if (SerialInterface.Instance == null)
            {
                 ConnectComboBox.SelectedIndex = 0;               
            }
        }

        //---------------------------------------------------------------------
        /// <summary>
        /// Called when a change made to combo box.
        /// Creates the serial interface if not already available and then sets
        /// up rest of form and serial interface.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (SerialInterface.Instance != null)
            {
               SerialInterface.Instance.Close();
            }

            if (ConnectComboBox.SelectedIndex > 0)
            {
                try
                {
                    // Create and Open the Controller port.
                    this.PortName = ConnectComboBox.SelectedItem.ToString();

                    // The chosen COM port is opened
                    SerialInterface Controler = SerialInterface.Create(this.PortName);

                    RelayControl = new ModuleInterface(Controler);

                    this.ConnectComboBox.Enabled = false;

                    this.StartScan.Enabled = true;
                    this.nuScanRate.Enabled = true;
                    EnableControls(true);
                    // Set the direction of Port pins
                    RelayControl.SetPortStates('B', ModuleInterface.DirectionState.OutputState,0x0);

                    // Now set lower 4 pins to ouputs to drive relays
                    RelayControl.SetPortStates('B', ModuleInterface.DirectionState.InputState, 0xF0);
                }

                catch
                {
                }
            }
        }

        //---------------------------------------------------------------------
        /// <summary>
        /// Sets up form controls to be enabled or disabled
        /// </summary>
        /// <param name="Enabled"></param>
        private void EnableControls(bool Enabled)
        {
            this.radNumber.Checked = Enabled;
            this.radCheck.Checked = Enabled;

            this.chkRelay1.Enabled = Enabled;
            this.chkRelay2.Enabled = Enabled;
            this.chkRelay3.Enabled = Enabled;
            this.chkRelay4.Enabled = Enabled;

            this.OpenRelay.Enabled = Enabled;
            this.CloseRelay.Enabled = Enabled;
            this.radCheck.Enabled = Enabled;
            this.radNumber.Enabled = Enabled;

            txtRelayNumber.Text = "";
            txtRelayNumber.Enabled = Enabled;

        }

        //---------------------------------------------------------------------
        /// <summary>
        /// Selects number option
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void radNumber_CheckedChanged(object sender, EventArgs e)
        {
            // Enables Text Box
            txtRelayNumber.Text = "";
            txtRelayNumber.Enabled = true;
            // Enables Open/close relay buttons
            this.OpenRelay.Enabled = true;
            this.CloseRelay.Enabled = true;
            // Disables Checkboxes
            this.chkRelay1.Enabled = false;
            this.chkRelay1.Checked = false;
            this.chkRelay2.Enabled = false;
            this.chkRelay2.Checked = false;
            this.chkRelay3.Enabled = false;
            this.chkRelay3.Checked = false;
            this.chkRelay4.Enabled = false;
            this.chkRelay4.Checked = false;
            
        }

        //---------------------------------------------------------------------
        /// <summary>
        /// Selects check box option
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void radCheck_CheckedChanged(object sender, EventArgs e)
        {
            // Disables Text Box
            txtRelayNumber.Text = "";
            txtRelayNumber.Enabled = false;

            // Disables Open/close relay buttons
            this.OpenRelay.Enabled = false;
            this.CloseRelay.Enabled = false;

            // Enables Checkboxes
            this.chkRelay1.Enabled = true;
            this.chkRelay1.Checked = false;
            this.chkRelay2.Enabled = true;
            this.chkRelay2.Checked = false;
            this.chkRelay3.Enabled = true;
            this.chkRelay3.Checked = false;
            this.chkRelay4.Enabled = true;
            this.chkRelay4.Checked = false;
        }

        //---------------------------------------------------------------------
        /// <summary>
        /// Closes whatever relay set in text box
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CloseRelay_Click(object sender, EventArgs e)
        {
            // Checks check box for relay closed
            int RelayNumber;

            if (int.TryParse(this.txtRelayNumber.Text, out RelayNumber))
            {
                switch (RelayNumber)
                {
                    case 1: this.chkRelay1.Checked = true;
                        break;
                    case 2: this.chkRelay2.Checked = true;
                        break;
                    case 3: this.chkRelay3.Checked = true;
                        break;
                    case 4: this.chkRelay4.Checked = true;
                        break;
                    default:
                        break;
                }
                // Sends Close command for relay value in text box
                RelayControl.CloseRelay(RelayNumber);
            }
        }

        //---------------------------------------------------------------------
        /// <summary>
        /// Opens whatever relay set in text box
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OpenRelay_Click(object sender, EventArgs e)
        {
            int RelayNumber;

            if (int.TryParse(this.txtRelayNumber.Text, out RelayNumber))
            {
                switch (RelayNumber)
                {
                    case 1: this.chkRelay1.Checked = false;
                        break;
                    case 2: this.chkRelay2.Checked = false;
                        break;
                    case 3: this.chkRelay3.Checked = false;
                        break;
                    case 4: this.chkRelay4.Checked = false;
                        break;
                    default:
                        break;
                }
                // Sends Open command for relay value in text box
                RelayControl.OpenRelay(RelayNumber);
            }
        }

        //----------------------------------------------------------------------
        /// <summary>
        /// Clears (opens) all relays by unchecking all check boxes
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ClearRelays_Click(object sender, EventArgs e)
        {            
            // UnChecks all check boxes
            this.chkRelay1.Checked = false;
            this.chkRelay2.Checked = false;
            this.chkRelay3.Checked = false;
            this.chkRelay4.Checked = false;
        }

        //---------------------------------------------------------------------
        /// <summary>
        /// Open/closes relay 1 when checkbox 1 is changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void chkRelay1_CheckedChanged(object sender, EventArgs e)
        {
            if (chkRelay1.Checked == true)
            {
                // Sends Close command for relay value in text box
                RelayControl.CloseRelay(1);
            }
            else
            {
                // Sends Open command for relay value in text box
                RelayControl.OpenRelay(1);
            }
        }

        //---------------------------------------------------------------------
        /// <summary>
        /// Open/closes relay 2 when checkbox 2 is changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void chkRelay2_CheckedChanged(object sender, EventArgs e)
        {
            if (chkRelay2.Checked == true)
            {
                // Sends Close command for relay value in text box
                RelayControl.CloseRelay(2);
            }
            else
            {
                // Sends Open command for relay value in text box
                RelayControl.OpenRelay(2);
            }
        }

        //---------------------------------------------------------------------
        /// <summary>
        /// Open/closes relay 3 when checkbox 3 is changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void chkRelay3_CheckedChanged(object sender, EventArgs e)
        {
            if (chkRelay3.Checked == true)
            {
                // Sends Close command for relay value in text box
                RelayControl.CloseRelay(3);
            }
            else
            {
                // Sends Open command for relay value in text box
                RelayControl.OpenRelay(3);
            }
        }

        //---------------------------------------------------------------------
        /// <summary>
        /// Open/closes relay 4 when checkbox 4 is changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void chkRelay4_CheckedChanged(object sender, EventArgs e)
        {
            if (chkRelay4.Checked == true)
            {
                // Sends Close command for relay value in text box
                RelayControl.CloseRelay(4);
            }
            else
            {
                // Sends Open command for relay value in text box
                RelayControl.OpenRelay(4);
            }
        }

        //---------------------------------------------------------------------
        /// <summary>
        /// Starts the reading of input port.  Changes text on button, and
        /// starts/stops timer.  If timer is started makes first reading of
        /// port.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void StartScan_Click(object sender, EventArgs e)
        {
            byte[] MsgBytes = new byte[2];

            if (StartScan.Text == "Start Scan")
            {
                ReceiveDataHandler();
                StartScan.Text = "Stop Scan";
                // Starts Timer
                ScanTimer.Start();

                // Instigates first read request 
            }
            else
            {
                StartScan.Text = "Start Scan";
                // Stop Timer
                ScanTimer.Stop();
            }
        }

        //---------------------------------------------------------------------
        /// <summary>
        /// Closes form when exit selected from menu
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        //---------------------------------------------------------------------
        /// <summary>
        /// Changes timer interval when spinner changes
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void nuScanRate_ValueChanged(object sender, EventArgs e)
        {
            this.ScanTimer.Interval = (int)this.nuScanRate.Value;
        }

        //---------------------------------------------------------------------
        /// <summary>
        /// Called when Timer Expires
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ScanTimer_Tick(object sender, EventArgs e)
        {
            byte[] MsgBytes = new byte[2];

            ReceiveDataHandler();

            if (this.StartScan.Text == "Stop Scan")
            {
                ScanTimer.Start();
            }
        }
    }
}
