﻿/* **************************************************************************
 * File Name:  SerialInterface.cs
 * Version  :  1.1
 * Author   :  Philip Allen
 * Date     :  8 May 2009
 * Updates  :  None
 ****************************************************************************/
using System;
using System.Collections.Generic;
using System.Linq;
using System.IO;
using System.IO.Ports;
using System.Text;

namespace SerialComms
{
    class SerialInterface
    {
        #region PrivateMemebers
        private SerialPort RelayPort;
        private const int DefaultBaudRate = 9600;
        protected static SerialInterface ControlInstance = null;

        #endregion

        //---------------------------------------------------------------------
        /// <summary>
        /// Returns reference to serial Port
        /// </summary>
        public static SerialInterface Instance
        {
            get { return ControlInstance; }
        }

        //---------------------------------------------------------------------
        /// <summary>
        /// Closes port
        /// </summary>
        /// <param name="ComPortName"></param>
        public void Close()
        {
            Dispose (true);
        }

        #region Constructors

        //---------------------------------------------------------------------
        /// <summary>
        /// Main constructor but can only be accessed from the Create function.
        /// This allows only one port to be created.
        /// </summary>
        /// <param name="ComPortName"></param>
        protected SerialInterface (string  ComPortName)
        {
            // Create Port
            RelayPort = new SerialPort(ComPortName, DefaultBaudRate,
                                                  Parity.None, 8, StopBits.One);
            // Create and Open the COM port.
             RelayPort.Open();

            // Check Port is available
            if (!RelayPort.IsOpen)
            {
                throw new Exception("Port is not open");
            }
        }

        //---------------------------------------------------------------------
        /// <summary>
        /// Checks if port is already open, if not creates a new instance, 
        /// otherwisw throws an exception
        /// </summary>
        /// <param name="ComPortName"></param>
        /// <returns></returns>
        public static SerialInterface Create(string ComPortName)
        {
            if (ControlInstance == null)
            {
                ControlInstance = new SerialInterface(ComPortName);
            }
            else
            {
                throw new Exception("Port already exists");
            }

            return ControlInstance;
        }

       #endregion

        //----------------------------------------------------------------------
        /// <summary>
        /// Send data
        /// </summary>
        public void Send(byte[] Packet)
        {
            {
                try
                {
                    // Writes 2 bytes of data to com port
                    if (RelayPort.IsOpen)
                    {
                        RelayPort.Write(Packet,0,2);
                    }
                }
                catch
                {
                    System.Diagnostics.Debug.WriteLine(
                        "Control.Send(): Failed to send message");
                }
            }
        }

       //----------------------------------------------------------------------
       /// <summary>
       /// Provides interface to a serial port read
       /// </summary>
       /// <param name="ReadBytes"></param>
       public void Read(out byte[] ReadBytes)
       {
           ReadBytes = new byte[2];
           RelayPort.Read(ReadBytes, 0, 2);
       }

       //----------------------------------------------------------------------
       /// <summary>
       /// Closes down application cleanly
       /// </summary>
       /// <param name="Disposing"></param>
       private void Dispose (bool Disposing)
       {
           if (Disposing)
           {
             if (RelayPort.IsOpen)
             {
               RelayPort.Close();
             }
           }
       }
    }
}
